#import "MyDocument.h"
#include "OpenGL/gl.h"
#include "SpinningTriangle.h"

/* override canBecomeKeyWindow for NSWindow so that even the borderless window is willing to become key */
@interface EBGFKeyWindow: NSWindow {
}

@end

@implementation EBGFKeyWindow

- (BOOL)canBecomeKeyWindow
{
	return YES;
}

@end

/* document implementation - handles switching into and out of full screen mode, includes model (a spinning triangle) alongside view as a completely temporary measure */

@implementation MyDocument

- (NSString *)windowNibName
{
	// Override returning the nib file name of the document; in this case the only one I have
	return @"MyDocument";
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
	[super windowControllerDidLoadNib:aController];

	// ensure content is always 4:3
	fullScreen = NO;
	[targetWindow setContentAspectRatio:NSMakeSize(4, 3)];

	// set the GL delegate
	SpinningTriangle *ST = [[SpinningTriangle alloc] init];
	[glView setDelegate:ST];
	[ST release];
}

- (NSRect)getTargetFrameForScreen:(NSScreen *)screen
{
	NSRect screenFrame = [screen frame];
	int screenWidth = (int)screenFrame.size.width;
	int screenHeight = (int)screenFrame.size.height;
	int targetWidth, targetHeight;

	if(screenWidth*3 != screenHeight*4)
	{
		if(screenWidth*3 > screenHeight*4)
		{
			// max height, work out new width
			targetWidth = (screenHeight*4)/3;
			targetHeight = screenHeight;
		}
		else
		{
			// max width, wortk out new height
			targetHeight = (screenWidth*3)/4;
			targetWidth = screenWidth;
		}
	}
	else
	{
		targetWidth = screenWidth;
		targetHeight = screenHeight;
	}

	NSRect targetFrame;
	targetFrame.size.width = targetWidth;
	targetFrame.size.height = targetHeight;
	targetFrame.origin.x = (screenFrame.size.width - targetFrame.size.width)*0.5;
	targetFrame.origin.y = (screenFrame.size.height - targetFrame.size.height)*0.5;

	return targetFrame;
}

- (void)toggleFullScreen:(NSMenuItem *)menuItem
{
	if(!fullScreen)
	{
		// get screen that will be used for the full-screen display, create a borderless window the same size as the current content frame
		NSScreen *targetScreen = [targetWindow screen];
		NSView *content = [targetWindow contentView];

		fullScreenWindow = [[EBGFKeyWindow alloc] 
								initWithContentRect:[targetWindow contentRectForFrameRect:[targetWindow frame]]
								styleMask:NSBorderlessWindowMask
								backing:NSBackingStoreBuffered
								defer:NO
								screen:targetScreen];

		// switch the content view from targetWindow to the newly created window
		[content retain];
		[targetWindow setContentView:nil];
		[fullScreenWindow setContentView:content];
		[content release];

		// now that the content has changed to the fullScreenWindow, draw a new frame (for OpenGL purposes) and make new window key and in front, with me as a delegate
		[fullScreenWindow makeKeyAndOrderFront:nil];
		[fullScreenWindow setDelegate:self];

		// hide old window
		[targetWindow orderOut:nil];

		// use Quartz to capture the display of interest - will now have blank screen with fullScreenWindow in front, but the wrong size
		int windowLevel = CGShieldingWindowLevel();
		NSNumber *screenNumber = [[[targetWindow screen] deviceDescription] objectForKey:@"NSScreenNumber"];
		CGDisplayCapture(screenNumber ? [screenNumber unsignedLongValue] : kCGDirectMainDisplay);
		[fullScreenWindow setLevel:windowLevel];

		// get fullScreenWindow to resize to full screen
		[fullScreenWindow setFrame:[self getTargetFrameForScreen:targetScreen] display:YES animate:YES];

		// set fullScreen flag
		fullScreen = YES;
	}
	else
	{
		// release the captured screen
		NSNumber *screenNumber = [[[fullScreenWindow screen] deviceDescription] objectForKey:@"NSScreenNumber"];
		CGDisplayRelease(screenNumber ? [screenNumber unsignedLongValue] : kCGDirectMainDisplay);

		// restore normal window, resize the fullScreenWindow to the content frame for the desktop window
		[fullScreenWindow setFrame:[targetWindow contentRectForFrameRect:[targetWindow frame]] display:YES animate:YES];

		// switch the content view from the fullScreenWindow back to the ordinary window
		NSView *content = [fullScreenWindow contentView];
		[content retain];
		[fullScreenWindow setContentView:nil];
		[targetWindow setContentView:content];
		[content release];

		// redraw and make the ordinary window front and key
		[targetWindow makeKeyAndOrderFront:nil];

		// release the fullscreen window
		[fullScreenWindow release]; fullScreenWindow = nil;

		// set fullscreen flag
		fullScreen = NO;
	}
}

@end
